#include<linux/module.h>
#include<linux/vmalloc.h>
#include<linux/slab.h>      

#define NUMBER_OF_ELEMENTS 20
#define MEMORY_SIZE NUMBER_OF_ELEMENTS*sizeof(int)

static int *first_array, *second_array;


static int __init mallocmod_init(void)
{
	first_array = (int *)vmalloc(MEMORY_SIZE);
	if(IS_ERR(first_array)) {
		pr_alert("Error allocating memory with the use of vmaloc(): %ld\n",PTR_ERR(first_array));
		return -ENOMEM;
	}
	pr_notice("The address in first_array: %p\n",first_array);

	second_array = (int *)kmalloc(MEMORY_SIZE,GFP_KERNEL);
	if(IS_ERR(second_array)) {
		pr_alert("Error allocating memory with the use of vmaloc(): %ld\n",PTR_ERR(second_array));
		return -ENOMEM;
	}
	pr_notice("The address in second_array: %p\n",second_array);
	
	return 0;
}

static void __exit mallocmod_exit(void)
{
	int i;
	
	if(second_array) {
		pr_notice("second_array content:\n");
		for(i=0;i<NUMBER_OF_ELEMENTS;i++)
			pr_notice("%d ",second_array[i]);
		pr_notice("\n");
		kfree(second_array);
	}

	if(first_array) {
		pr_notice("first_array content:\n");
		for(i=0;i<NUMBER_OF_ELEMENTS;i++)
			pr_notice("%d ",first_array[i]);
		pr_notice("\n");
		vfree(first_array);
	}
}

module_init(mallocmod_init);
module_exit(mallocmod_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Arkadiusz Chrobot <a.chrobot@tu.kielce.pl>");
MODULE_DESCRIPTION("A module demonstrating the usage of different kernel memory allocators.");
MODULE_VERSION("1.0");
